Package: microbiomeExplorer
Type: Package
Title: Microbiome Exploration App
Version: 1.23.0
Date: 2022-09-01
Authors@R: c(
    person("Joseph", "Paulson", , "paulson.joseph@gene.com", role = c("aut")),
    person("Janina", "Reeder", , "reederj1@gene.com", role = c("aut", "cre")),
    person("Mo", "Huang", role = c("aut")),
    person("Genentech", role = c("cph", "fnd"))
    )
Description: The MicrobiomeExplorer R package is designed to facilitate the analysis and visualization of marker-gene survey feature data. 
  It allows a user to perform and visualize typical microbiome analytical workflows either through the command line or an interactive 
  Shiny application included with the package. In addition to applying common analytical workflows the application enables automated 
  analysis report generation.
License: MIT + file LICENSE
Imports: shinyjs (>= 2.0.0), shinydashboard, shinycssloaders,
        shinyWidgets, rmarkdown (>= 1.9.0), DESeq2, RColorBrewer,
        dplyr, tidyr, purrr, rlang, knitr, readr, DT (>= 0.12.0),
        biomformat, tools, stringr, vegan, matrixStats, heatmaply, car,
        broom, limma, reshape2, tibble, forcats, lubridate, methods,
        plotly (>= 4.9.1)
Depends: shiny, magrittr, metagenomeSeq, Biobase
Suggests: V8, testthat (>= 2.1.0)
DeploySubPath: microbiomeExplorer
biocViews: Classification, Clustering, GeneticVariability,
        DifferentialExpression, Microbiome, Metagenomics,
        Normalization, Visualization, MultipleComparison, Sequencing,
        Software, ImmunoOncology
Encoding: UTF-8
RoxygenNote: 7.2.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/microbiomeExplorer
git_branch: devel
git_last_commit: 2398fa4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:46:19 UTC; biocbuild
Author: Joseph Paulson [aut],
  Janina Reeder [aut, cre],
  Mo Huang [aut],
  Genentech [cph, fnd]
Maintainer: Janina Reeder <reederj1@gene.com>
