# Generated by roxygen2: do not edit by hand

export(addFeatData)
export(addPhenoData)
export(aggFeatures)
export(aggregationTabUI)
export(calculatePCAs)
export(computeDistMat)
export(corrAnalysisUI)
export(corrFeature)
export(corrPhenotype)
export(dataInputUI)
export(diffAnalysisUI)
export(extendPhenoData)
export(featureAnalysisUI)
export(featureTableUI)
export(filterByPheno)
export(filterMEData)
export(interAnalysisUI)
export(intraAnalysisUI)
export(longAnalysisUI)
export(makeQCPlot)
export(normalizeData)
export(phenotypeTableUI)
export(plotAbundance)
export(plotAlpha)
export(plotAvgAbundance)
export(plotBeta)
export(plotHeatmap)
export(plotLongFeature)
export(plotSingleFeature)
export(plotlyHistogram)
export(plotlySampleBarplot)
export(readData)
export(replaceWithUnknown)
export(reportListUI)
export(rollDownFeatures)
export(runDiffTest)
export(runMicrobiomeExplorer)
import(metagenomeSeq)
import(shiny)
import(shinydashboard)
importFrom(Biobase,"fData<-")
importFrom(Biobase,"featureData<-")
importFrom(Biobase,"pData<-")
importFrom(Biobase,"phenoData<-")
importFrom(Biobase,fData)
importFrom(Biobase,pData)
importFrom(magrittr,"%>%")
importFrom(metagenomeSeq,"normFactors<-")
importFrom(metagenomeSeq,MRcounts)
importFrom(metagenomeSeq,MRtable)
importFrom(metagenomeSeq,aggTax)
importFrom(metagenomeSeq,biom2MRexperiment)
importFrom(metagenomeSeq,cumNorm)
importFrom(metagenomeSeq,cumNormStatFast)
importFrom(metagenomeSeq,fitFeatureModel)
importFrom(metagenomeSeq,loadMeta)
importFrom(metagenomeSeq,loadPhenoData)
importFrom(metagenomeSeq,newMRexperiment)
importFrom(metagenomeSeq,normFactors)
importFrom(methods,is)
importFrom(rlang,":=")
